$!  SERVERCONFIG.COM
$!  19-AUG-1994, David Mathog, Biology Division, Caltech
$!
$!  This procedure is used to simplify connecting remote X11 servers
$!  to a VMS machine.  It accepts five arguments, the first three of which are 
$!  mandatory.  These are:
$!
$!1 TRANSPORT     Options:  CURRENT, TCPIP, DECNET  (later, LAT, ?)
$!2 NODENAME      The name of the X11 server correct for the current 
$!                transport (in some cases the number will work too).
$!
$!                "CURRENT CURRENT" is used to do something without
$!                changing the transport any.
$!
$!                Screen numbers can be added by (optionally) using the
$!                unix style nodename> = node:head.screen
$!
$!                Currently (DW/motif 1.2) HEAD is not supported and will
$!                be ignored.
$!
$!                It's possible to change just the screen by passing in 
$!                a nodename like:  CURRENT:0.1
$!
$!3 DOWHAT        Abbreviations for various common options.  Only a 
$!                SINGLE one of these may be started.  When it exits
$!                this .COM exits too.  NONE can be specified if all you
$!                want to do is aim the display someplace else.
$!
$!    -DECwindows/Motif applications-
$!                DECW$SESSION     DECW session manager, but NO window manager
$!                DECW$MWM         MWM window manager
$!
$!                DECW$BOOKREADER
$!                DECW$CALC
$!                DECW$CALENDAR
$!                DECW$CARDFILER
$!                DECW_CDAVIEWER
$!                DECW$CLOCK
$!                DECW_DECTERM
$!                DECW_EVE
$!                DECW_FILEVIEW
$!                DECW$MAIL
$!                DECW$NOTEPAD
$!                DECW$PAINT
$!                DECW$PUZZLE
$! 
$!    -DEC layered applications -
$!                DECWRITE
$!                DECCHART$MOTIF
$!                DECISION$ECALC
$!
$!    -Shareware etc -
$!                MOSAIC
$!                NEDIT
$!                NENTREZ
$!                TED
$!                WPI
$!                XFIG
$!                XV
$!      Note: GS won't work this way since it demands a valid SYS$INPUT.
$!    -Special
$!                NONE   - use this to change the default screen, but don't
$!                         run anything.
$!
$!4 MODMAP        Modmap to apply to modify keyboard for the server.
$!                Options are:
$!                CURRENT       (don't change)
$!                MACX_EXTENDED   (MacX extended keyboard)
$!                MICROX_WIN      (MicroX Win with keyboard file SAFDEC.KBD)
$!
$!5 VUE$PROFILE   If specified as anything other than a null string, use
$!                that VUE$PROFILE instead of whatever is in VUE$LIBRARY
$!                or sys$login.  Currently valid options are:
$!                ""             (Look in VUE$LIBRARY, then SYS$LOGIN)
$!                DEFAULT        (Default.vue$dat in serverconfig)
$!                MACX           (Appropriate for use with MacX)
$!                MACX_SMALL     (Appropriate for use with MacX on small 
$!                               screens)
$!
$!6 OPTIONS       If present, these must be enclosed in "" and and single
$!                quotes must be replaced with double quotes.  These may
$!                be added to the command line which invokes any NON DECW$
$!                application.  Use these with GREAT care!!!
$!
$!   **************************************************
$!
$ ws := write sys$output
$! set ver
$!
$!  Do a full login if this is a network connection - otherwise
$!  various things will fail later on.  Also turn on all x11 logicals
$!  so that things like TED will run correctly
$!
$  IF F$MODE() .EQS. "OTHER"
$then
$ @sys$manager:sylogin/output=nla0: jam
$ file = F$SEARCH("SYS$LOGIN:LOGIN.COM")
$ if(file .nes. "")then @sys$login:login/output=nla0: jam
$ x11setup silent
$endif
$!
$!  Figure out where this .COM file is sitting, some other data files will
$!  be there too.
$!
$ procfile = F$Environment("PROCEDURE")
$ procdev  = F$Parse(procfile,,,"DEVICE","NO_CONCEAL")
$ procdir  = F$Parse(procfile,,,"DIRECTORY","NO_CONCEAL")
$ procloc  = procdev + procdir
$!
$!
$!  Verify that TRANSPORT is ok, otherwise STOP
$!
$  ptemp = F$EDIT(P1,"UPCASE,COLLAPSE")
$  TRANSPORT = ""
$  if(PTEMP .eqs. "CURRENT")then TRANSPORT = "CURRENT"
$  if(PTEMP .eqs. "TCPIP")then   TRANSPORT = "TCPIP"
$  if(PTEMP .eqs. "DECNET")then  TRANSPORT = "DECNET"
$!for later  if(PTEMP .eqs. "LAT")then TRANSPORT = "LAT"
$if (TRANSPORT .eqs. "")
$then
$  ws "The TRANSPORT parameter was incorrectly specified as >''PTEMP'<"
$  ws "Valid options are: CURRENT,TCPIP,and DECNET"
$  STOP
$endif
$!
$!  Verify that NODENAME is ok, otherwise STOP.  Here OK just means nonblank
$!
$  NODENAME = F$EDIT(P2,"UPCASE,COLLAPSE")
$if (NODENAME .eqs. "")
$then
$  ws "The NODENAME parameter was incorrectly specified as >''PTEMP'<"
$  ws "Valid options are: CURRENT or the name of your X11 server appropriate"
$  ws " for the ''TRANSPORT' transport"
$  STOP
$endif
$!
$!  Ok, now extract SCREEN and HEAD, if any
$!
$  iscolon  = F$LOCATE(":",NODENAME)
$  islength = F$LENGTH(NODENAME)
$  if(iscolon .eq. islength)
$then
$  screen = ""
$  head   = ""
$  goto checkcurrent
$else
$  HEADSCREEN  = F$EXTRACT(ISCOLON+1,ISLENGTH - ISCOLON,NODENAME)
$  NODENAME = F$EXTRACT(0,ISCOLON,NODENAME) 
$endif
$  isperiod = F$LOCATE(".",HEADSCREEN)
$  islength = F$LENGTH(HEADSCREEN)
$  if(isperiod .eq. islength)
$then
$  ws "The NODENAME parameter was incorrectly specified as >''PTEMP'<"
$  ws "The portion :HEAD.SCREEN is invalid - no . is present"
$  STOP
$else
$  head    = F$EXTRACT(0,ISPERIOD, HEADSCREEN)
$  screen  = F$EXTRACT(ISPERIOD + 1, ISLENGTH - ISPERIOD, HEADSCREEN)
$endif
$if (SCREEN .nes. "") then screen = "/screen = ''screen'"
$if (HEAD   .nes. "") then head   = "/head   = ''head'"
$!
$!  So far so good.
$!  If nodename is CURRENT make sure that one exists
$!    and also extract it's parameters
$!  If nodename is NOT CURRENT create one.
$!
$Checkcurrent:
$!
$!  figure out if a display exists, and if so, what it's parameters
$!  are.
$!
$ define/user/nolog sys$output sys$login:zapmenow.txt
$ show display
$  if(.NOT. $STATUS)
$then
$  DIS_EXISTS = 0
$  delete sys$login:zapmenow.txt.
$else
$  DIS_EXISTS = 1
$ open/read tfil: sys$login:zapmenow.txt
$! a blank line
$ read/error=bad_display/end=bad_display  tfil: STRING
$!
$! the device name
$!
$ read/error=bad_display/end=bad_display  tfil: STRING
$ islength = F$LENGTH(STRING)
$ STRING   = F$EXTRACT(15, ISLENGTH - 15,STRING)
$ iscolon  = F$LOCATE(":",STRING)
$ islength = F$LENGTH(STRING)
$ if(iscolon .eq. islength) then goto bad_display
$ DIS_DEVICE = F$EXTRACT(0,ISCOLON+1,STRING)
$!
$!  Node
$!
$ read/error=bad_display/end=bad_display  tfil: STRING
$ islength = F$LENGTH(STRING)
$ DIS_NODE = F$EXTRACT(15, ISLENGTH - 15,STRING)
$!
$!  Transport
$!
$ read/error=bad_display/end=bad_display  tfil: STRING
$ islength = F$LENGTH(STRING)
$ DIS_TRANSPORT = F$EXTRACT(15, ISLENGTH - 15,STRING)
$!
$!  Server
$!
$ read/error=bad_display/end=bad_display  tfil: STRING
$ islength = F$LENGTH(STRING)
$ DIS_SERVER = F$EXTRACT(15, ISLENGTH - 15,STRING)
$!
$!  Screen
$!
$ read/error=bad_display/end=bad_display  tfil: STRING
$ islength = F$LENGTH(STRING)
$ DIS_SCREEN = F$EXTRACT(15, ISLENGTH - 15,STRING)
$!
$! clean up
$!
$ close tfil:
$ delete sys$login:zapmenow.txt.
$ goto OutOfHere
$BAD_DISPLAY:
$ ws "DECW$DISPLAY is defined but values are invalid"
$ type sys$login:zapmenow.txt
$ del  sys$login:zapmenow.txt.
$ STOP
$ENDIF
$OutofHere:
$!
$!  Ok, now do whatever is appropriate
$!
$  if(NODENAME .nes. "CURRENT" .and. TRANSPORT .nes. "CURRENT") -
   then goto set_transport
$!
$! doing a CURRENT operation, display must already exist
$!
$  if .not. DIS_EXISTS 
$then
$  ws "No display exists - so you may not specify CURRENT for"
$  ws "The NODENAME parameter was incorrectly specified as >''PTEMP'<"
$  ws "The portion :HEAD.SCREEN is invalid - no . is present"
$  STOP
$endif
$!
$  if (SCREEN .eqs. "") then goto AFTER_TRANSPORT
$  set display 'screen'
$! 'head' too, if DECwindows ever supports it
$  goto AFTER_TRANSPORT
$SET_TRANSPORT:
$  set display/create/node='NODENAME'/transport='TRANSPORT' 'screen'
$! add 'head' if they ever support it...
$  HOLD_=$STATUS
$  if (.not. HOLD_)
$then
$  ws "Could not set display from this VMS system to your server"
$  ws "The error code was ''HOLD_'"
$  ws "The most likely errors are:"
$  ws "  1.  Invalid nodename, due to typos"
$  ws "  2.  Invalid nodename, due to wrong name for this transport"
$  ws "  3.  Your X11 server rejected the connect request"
$  STOP
$endif
$AFTER_TRANSPORT:
$!
$!  At this point there should be a valid transport.
$!  Extract DOWHAT, MODMAP, VUE$PROFILE, and OPTIONS
$!
$   DOWHAT  = F$EDIT(P3,"UPCASE,COLLAPSE")
$   MODMAP  = F$EDIT(P4,"UPCASE,COLLAPSE")
$   CUSTVUE = F$EDIT(P5,"UPCASE,COLLAPSE")
$   OPTIONS = P6
$!
$!  verify that the modmap file exists, if specified
$!
$ if (MODMAP .nes. "CURRENT" .and. MODMAP .nes. "")
$then
$!  procname = F$ENVIRONMENT("PROCEDURE")
$!  modfile = F$PARSE(procname,,,"DEVICE")
$  modfile = procloc + MODMAP + ".DAT"
$  if (F$SEARCH(MODFILE) .eqs. "")
$then
$  ws "The modmap file that you specified >''MODMAP'< does not exist"
$  ws "(Hint: Check your spelling)"
$  STOP
$endif
$else
$  modfile = ""
$endif
$!
$!  verify that the vue$profile exists, if specified
$!
$ if (CUSTVUE .nes. "")
$then
$  vuefile = procloc + CUSTVUE + ".VUE$DAT"
$  if (F$SEARCH(VUEFILE) .eqs. "")
$then
$  ws "The custom VUE$PROFILE that you specified >''CUSTVUE'< does not exist"
$  ws "Here is a list of known VUE$PROFILE files:"
$  dir/col=1 'procloc'*.vue$dat
$  ws "(Hint: Check your spelling)"
$  STOP
$endif
$ define vue$profile 'vuefile'
$endif
$!
$  if("''DOWHAT'" .eqs. "")
$then
$  ws "Error, no actions were specified for this transport."
$  STOP
$endif
$!
$!
$!  Do a modmap on the server if one is specified.  Have to use the
$!  delayed method, to be sure that it takes effect if decw$session is run
$!
$if ("''MODFILE'" .nes. "")
$then
$  spawn/nowait @serverconfig:delay_modmap 00:00:05.00 'modfile'
$endif
$!
$!  Actually do whatever is required
$!
$  POSITION = 0
$  ACTION=DOWHAT
$  call process "reserved" "''ACTION'" "''OPTIONS'"
$!
$AFTER_ACTION:
$!STOP
$EXIT
$!**************************************************
$!
$!
$! This subroutine peforms the desired action:
$!   DECW$* are run from SYS$SYSTEM
$!   DECW_* are treated as appropriate
$!   DEC*   are run from SYS$SYSTEM
$!   Anything else is treated as a foreign command
$!
$! parameters are:
$!
$!  P1  reserved for future use
$!  P2  action
$!  P3  options (if any)
$!
$PROCESS:  SUBROUTINE
$!   
$!
$!  the next one is useful when all you want to do is diddle
$!  around with the display
$!
$   if (P2 .eqs. "NONE")then ENDSUBROUTINE
$!
$!  ok, it must be a real action of some sort
$!
$   subaction = F$EXTRACT(0,5,P2)
$   doit = "STOP"
$   if(subaction .eqs. "DECW$")
$then 
$   doit = "run sys$system:''P2'"
$   if(F$SEARCH("SYS$SYSTEM:''P2'.EXE") .eqs. "")
$then
$   ws "The action you specified >''P2'< is not known"
$   ws "(Hint: Check your spelling)"
$   STOP
$endif
$endif
$   if(subaction .eqs. "DECW_")
$then
$   if(P2 .eqs. "DECW_CDAVIEWER")then doit = "VIEW/INTERFACE=DECWINDOWS"
$   if(P2 .eqs. "DECW_DECTERM")then doit =   "CREATE/TERMINAL=DECTERM/DETACH
$   if(P2 .eqs. "DECW_EVE")then doit =       "EDIT/TPU/DISPLAY=DECWINDOWS"
$   if(P2 .eqs. "DECW_FILEVIEW")then doit =  "RUN SYS$SYSTEM:VUE$MASTER"
$   if(doit .eqs. "STOP")
$then
$   ws "The action you specified >''P2'< is not known"
$   ws "(Hint: Check your spelling)"
$   STOP
$endif   
$endif
$   subaction = F$EXTRACT(0,3,P2)
$   if(subaction .eqs. "DEC" .and. doit .eqs. "STOP")then -
    doit = "RUN SYS$SYSTEM:''P2' ''P3'"
$!
$!  At this point DOIT should have been defined, if not, it will be STOP
$!  and so flag an error
$!
$   if(doit .eqs. "STOP")
$then
$   if(F$TYPE('P2') .eqs. "")
$then
$  ws "The action ''P2' is not defined"
$  ws "(Hint: Check your spelling)"
$  STOP 
$endif
$!
$  doit = "''P2' ''P3"
$endif
$!
$!  
$! old method...  spawn/nowait/input=nla0: 'doit'
$ 'doit'
$  if(.not. $STATUS)
$then
$  ws "The action ''doit' failed"
$  ws "(Hint: Check your spelling and the order of actions)"
$  STOP 
$endif
$ENDSUBROUTINE
