$!   VUE$TIMEOUT.com
$!
$!   David Mathog, 1-DEC-1994, Biology Division, Caltech
$!
$!   This can be used in DECwindows sessions that run under the session
$!   manager to kill off whole sessions after they are idle for more than
$!   the specified timeout interval.
$!
$!   If VUE$POPMESSAGE isn't also running then there will be no warning
$!   before disconnect!  So, if you use this, put both of these procedures
$!   into autostart, with vue$popmessage starting first.
$!
$!   This procedure uses decw$endsession on the console (as detected by a 
$!   LOCAL transport), but on remote X Servers it just kills all jobs on
$!   the X client (the VMS machine where this procedure is running).  It
$!   is difficult enough to tell which remote machine we're talking to so
$!   that no attempt is made to use decw$endsession on a case by case basis
$!
$!
$!*******************************************************************
$!
$!   A couple of constants.
$!
$!      W_Interval:  time to wait before popping up a warning box
$!      S_Interval:  time to wait after that before committing suicide
$!                   Note, if no VUE$POPMESSAGE is used then this interval
$!                   is zero - no point waiting for a respone if no prompt!
$!      D_Interval:  time to wait before checking again to see if a
$!                    duplicate vue$timeout is running.
$!
$!
$!    W_Interval   = "00:20:00"
$!    S_interval   = "00:02:00"
$    W_Interval   = "00:02:00"
$    S_interval   = "00:00:50"
$    I_interval   = "00:00:20"
$    D_interval   = "00:02:00"
$    line_feed[0,8] = 10
$    teststring :== "TEST"
$    old_teststring :== " "
$
$!
$!  Suppress output popup.  It should only come up when we check to
$!  see if popmessage is running - no way to the output from SHO PROCESS
$!
$    vue$suppress_output_popup
$!
$!
$!
$    username      =  f$getjpi("","USERNAME")
$    my_name       = "KILL_" + username
$!
$!   retrieve the PID of the POP_username.  Check that it applies to
$!   a currently running process.
$!
$    POP_PID = "BAD"
$    open/read/error=noPOP_PID tfil: vue$popup.pid
$    read/error=badPOP_PID/end=badPOP_PID  tfil:  POP_PID
$badPOP_PID:
$    close tfil:
$noPOP_PID:
$!
$!  See if we can retrieve a master_pid
$!
$if POP_PID .nes. "BAD"
$then
$  PMaster = f$getjpi(POP_PID,"MASTER_PID")
$  if .not. $STATUS then POP_PID = "BAD"
$endif
$!
$!  See if pop's master_pid is the same as ours
$!
$if POP_PID .nes. "BAD"
$then
$  if (PMaster .nes. f$getjpi("","MASTER_PID"))then POP_PID = "BAD"
$endif
$!
$!   try to change the name of this process to my_name
$!   if it fails, that means that one is already running. Don't want two!
$!   There could be two for two reasons:
$!     1.  Two rexec/rshells from the same remote terminal.
$!     2.  Two sessions running on different terminals.
$!   What we do here is try to set the name.  If it fails, we wait interval
$!   and try again.  Only if we succeed do we proceed to set up idle job
$!   monitoring.
$!             
$DUPLOOP:
$    set process/name='my_name'
$    if $STATUS then goto EXDUPLOOP
$    wait 'D_interval'
$    goto DUPLOOP
$EXDUPLOOP:
$!
$!
$!   Now figure out some PID's and set up the main idle job
$!   detection loop.
$!
$    my_PID        = f$getjpi("","PID")
$    slaughter_PID = f$getjpi("","MASTER_PID")
$    kill_pid      = "KILL_" + slaughter_pid
$!
$!  This next section works but isn't used in this release - too much of
$!  a mess determining when and when not to put the login screen up.
$!
$! $!
$! $!   prompt the user - do they want to put up a login screen after
$! $!   logout.
$! $!
$! $    vue$popup_confirm "If session times out put a login screen", -
$!      line_feed,-
$!      "up on this terminal?",line_feed,line_feed
$! $    vue$read maybe
$! $    if (maybe .eqs. "NO")
$! $then
$! $    mybyebye = "DEATH"
$! $else
$! $    mybyebye = "REBIRTH"
$! $endif
$ mybyebye = "DEATH"
$
$!
$!  Kill all privs except tmpmbx,netmbx, otherwise bad things 
$!  might happen
$!
$ set proc/priv=(NOCMKRNL,NOCMEXEC,NOSYSNAM,NOGRPNAM -
 ,NOALLSPOOL,NODETACH,NODIAGNOSE -
 ,NOLOG_IO,NOGROUP,NOACNT,NOPRMCEB,NOPRMMBX,NOPSWAPM,NOALTPRI -
 ,NOSETPRV,TMPMBX,NOWORLD,NOMOUNT -
 ,NOOPER,NOEXQUOTA,NETMBX,NOVOLPRO,NOPHY_IO,NOBUGCHK,NOPRMGBL -
 ,NOSYSGBL,NOPFNMAP,NOSHMEM -
 ,NOSYSPRV,NOBYPASS,NOSYSLCK,NOSHARE,NOGRPPRV,NOREADALL,NOSECURITY)
$!
$!   Store the current bufio, directioIO for all user processes
$!   (except this one, and the popmessage process)
$!
$!
$TOP:
$    call track_io 'My_PID' 'POP_PID'
$!
$!   wait awhile
$!
$    wait 'w_interval'
$!
$!   see if any IO has taken place in any process other than the current
$!   and the POP
$!
$    call track_io 'My_PID' 'POP_PID'
$!
$!  See if the teststrings match
$!
$if (teststring .nes. old_teststring)then goto TOP
$!
$!   Ok, looks like we're dealing with an idle process
$!   Put up a dialog box warning of impending doom - only
$!   if a popmessage process exists and it is SUSP.  If it
$!   isn't in SUSP then there is no way to force it to put up
$!   a message!
$!
$ DOIT = 0
$ IF POP_PID .nes. "BAD"
$THEN 
$ if(f$getjpi(POP_PID,"STATE") .eqs. "SUSP")then DOIT = 1
$ENDIF
$!
$ if DOIT
$THEN
$    open/write tfil: sys$login:vue$popup.message
$    write tfil: " "
$    write tfil: "                Warning!"
$    write tfil: " "
$    write tfil: "Idle session automatic logout in progress."
$    write tfil: " "
$    write tfil: "''w_interval' = time this session has been idle"
$    write tfil: "''S_interval' = time to logout"
$    write tfil: " "
$    write tfil: "(Click either button to cancel the logout)"
$    write tfil: " "
$    close tfil:
$!
$!   Now wake up the popmessage process.  It MUST have already
$!     been started, for instance, from autostart.
$!   Note that the it will delete all vue$popup.message files after it
$!     reads the top one.
$!
$    set process/resume/id='POP_PID'
$    wait 'I_interval'
$    call track_io 'My_PID' 'POP_PID'
$!
$!   Wait awhile to see if there is any activity in a process OTHER than
$!   this one and the popmessage one.  If the popmessage has been
$!   clicked by the user it may now be back in a SUSP state.  If so, then
$!   abort this suicide.  However, if it is NOT in SUSP we can't conclude
$!   much, since another window may have popped it up.  So to check that
$!   have to wait a while and then check IO.  We also keep checking the
$!   popmessage process to see if it has gone back into SUSP so that
$!   we can bop out one loop and so put up a warning message.
$!
$POP_WAIT_TOP:
$    if(f$getjpi(POP_PID,"STATE") .eqs. "SUSP")then goto TOP
$    wait 's_interval'
$!
$!   If we got here it might be curtains!
$!
$    call track_io 'My_PID' 'POP_PID'
$!
$!   User's last chance to have done something...
$!
$if (teststring .nes. old_teststring)then goto POP_WAIT_TOP
$endif
$!
$!
$!   "Time to die"
$!
$  define/user sys$output nla0:
$  show display/symbol
$  if(DECW$DISPLAY_TRANSPORT .eqs. "LOCAL")
$then
$   ess :== $sys$system:decw$endsession
$   ess -"noprompt"
$elseif
$    call slaughter "''my_pid'" "''slaughter_pid'" "''mybyebye'"
$endif
$ exit
$!
$!  subroutines
$!
$!  TRACK_IO.  Note the current values of Buffered and Direct IO
$!  for all processes that have a master_pid that is NOT a batch job.
$!  Except for the current process (PID in P1) and the
$!  POP_PID (pop process PID in P2).
$!
$TRACK_IO: SUBROUTINE
$  current = ""
$  old_teststring == teststring
$  teststring :== "TEST"
$TR_TOP:
$  pid = F$pid(current)
$  if (pid .eqs. "") then EXIT
$  if (pid .eqs. P1) then goto TR_TOP
$  if (pid .eqs. P2) then goto TR_TOP
$!
$!  Make sure Master_PID is NOT a batch job.
$!
$  MASTER_PID = F$GETJPI(pid,"MASTER_PID")
$  MMODE      = F$GETJPI(MASTER_PID,"MODE")
$  if ("BATCH" .eqs. MMODE)then goto TR_TOP
$!
$! Ok, add it to the test string
$!
$  teststring :== 'teststring''F$GETJPI(pid,"BUFIO")''F$GETJPI(pid,"DIRIO")'
$  goto TR_TOP
$!
$ EXIT
$ ENDSUBROUTINE
$!
$!   Slaughter kills all processess OTHER than the one passed in by P1
$!   (the current process), and P2 (the master process) that are NOT
$!   subprocesses of a batch job.  If the user has another interactive 
$!   session going it may get ugly.  
$!   Teach THAT user not to leave a terminal unattended!
$
$SLAUGHTER: SUBROUTINE
$!
$!
$  current = ""
$SL_TOP:
$!
$!  Get the PID of the next possible terminal
$!
$  pid = F$pid(current)
$!
$!  If all have been checked, go and kill the one that ran this
$!
$  if (pid .eqs. "") then goto FINAL_EXIT  !end of list?
$!
$!  Don't kill the current process or its master this way
$!
$  IF (P1 .eqs. PID)then goto SL_TOP
$  IF (P2 .eqs. PID)then goto SL_TOP
$!
$!  Make sure Master_PID for the PID in question is NOT a batch job, then
$!    kill it.
$!
$  MASTER_PID = F$GETJPI(pid,"MASTER_PID")
$  MMODE      = F$GETJPI(MASTER_PID,"MODE")
$  if ("BATCH" .eqs. MMODE)then goto SL_TOP
$!
$!  Go not gently into that good night...
$!
$  stop/id='pid'
$!
$!  Go look for another victim
$!
$  goto SL_TOP
$!
$!  Say bye-bye
$!
$FINAL_EXIT:
$!
$! REBIRTH/DEATH are not implemented yet.
$! For now, figure out if this is running on the console, and if
$!  so, go out with the decw$endsession.
$! Otherwise, kill P2 and P1 and hope for the best
$!
$  stop/id='P2'
$  stop/id='P1'
$  stop/id=0
$!
$! The procedure should never get to this point.
$!
$ exit
$ ENDSUBROUTINE
